%============================================================================
% Field reduction for propagation past building rows
% (diffraction calculation for multiple screen and incident cylindrical wave)
%
% Input parameters:
%
%   Nmax: maximum number of screens (200)
%   limq: maximum value of q to evaluate coefficients I(N,q,2). (170)
%
%   gc=[0.494 0.370 0.247 0.123]: vector type for multiple gc
%   ( minimum gc should not be less than 0.1, or it will be changed to 0.1
%     automatically to avoid underflow of (2*sqrt(j*pi)*gc)^q for large q)
%
%   * gc=y0/sqrt(lambda*d)
%============================================================================
clear
%----------------------------- input parameters -----------------------------

Nmax=200;
limq=170;

gc=[2.7557 1.2247 0.0245 -1.2247 -2.7557]; % gc for f=900MHz, d=50m, and
                                           % y0=[11.25 5 0.1 -5 -11.25]

%-------------- check minimum gc value to avoid underflowflow ---------------

[gcmin,ind]=min(abs(gc));
while gcmin<0.1,
    str=sprintf('\nThe minimum gc %e is changed to 0.1',gcmin);
    str=sprintf('%s to avoid underflow\n',str);
    disp(str);
    gc(ind)=0.1*sign(gc(ind));
    if gc(ind)==0, gc(ind)=0.1; end
    [gcmin,ind]=min(abs(gc));
end
%------------------ calculate I(N,1,2)/q! for N=1,..,Nmax -------------------

I2=zeros(Nmax,limq);
I20=ones(Nmax+1,1);

for n=2:Nmax+1,                 % I20(n+1)=I(n,0,2)=1/((n+1)^(3/2))
    I20(n)=n.^-1.5;
end                             % I(n,0,2)/q!=I(n,0,2)/0!=I(n,0,2)

aux=1/(2*sqrt(pi));
I2(1,1)=aux/2;                  % I2(1,1) is I(N=M-1=1,q=1,beta=2)
for N=2:Nmax,
    Sum=0;
    for n=1:N,
        Sum=Sum+1/(n*(N+1-n))^1.5;
    end
    I2(N,1)=(aux/2)*Sum;
end

%----------------- calculate I(N,2,2)/q! for N=1,...,Nmax -------------------

I2(1,2)=1/(8*sqrt(2));          % I2(1,2)=I(N=M-1=1,q=2,beta=2)
                                % I(1,0,2)=1/(2^1.5);
                                % I(1,2,2)=1/(2*2)*I(1,0,2)+0
for N=2:Nmax,
    Sum=0;
    for n=1:N-1,                % n=(beta-1) to N-1
        Sum=Sum+I2(n,1)/sqrt(N-n);
    end
    I2(N,2)=(N/2*I20(N+1)+aux*Sum)/(N+1);
end
I2(:,2)=I2(:,2)/2;              % I2/q!  for q=2

%------------ calculate I(N,q,2)/q! for N=1,...,Nmax, q=3,...,lmiq ----------

for q=3:limq
    I2(1,q)=1/4 * I2(1,q-2)/q;  % for N=1
    for N=2:Nmax
        Sum=0;
        for n=1:N-1             % n=(beta-1) to N-1
            Sum=Sum+I2(n,q-1)/sqrt(N-n);
        end
        I2(N,q)=(N/2*I2(N,q-2) + aux*Sum)/(q*(N+1));
    end
    str=sprintf('I(N,q,beta=2) for q=%d of %d completed',q,limq);
    disp(str);
end

%--------------------- calculate magnitude of H field -----------------------

aux=2*sqrt(j*pi).*gc(:);
Q=zeros(length(gc),Nmax);       % gc can be a vector type
for N=1:Nmax,
    Sum=I20(N+1);               % for q=0; I(N,0,2)
    for q=1:limq,
        Sum=Sum+I2(N,q).*aux.^q;
    end
    Q(:,N)=sqrt(N+1)*abs(Sum);  % reduction factor of H(N+1) field
end

loglog(2:N+1,Q);    % field reduction factor vs M (or N+1,but not N)
ax=axis;
axis([1 N ax(3) ax(4)])
xlabel('M')
ylabel('Q_{M}')
title('Rooftop Field Reduction for Incident Cylindrical Wave')

%---------------------------- end of program --------------------------------