%============================================================================
%  Path loss as a function of distance x in LOS street canyon
%  (6 ray model for zt, zr variable)
%
% Input parameters
%
%          f: frequency (900MHz)
%         h1: base staion antenna height (10m)
%         h2: mobile antenna height (1.8m)
%         zt: transmitter z-coordinate (8.0m)
%         zr: receiver z-coordinate (8.0m)
%      width: width of street canyon (30.0 m)
%         eg: dielectric constant of ground (15.0)
%         ew: dielectric constant of walls (6.0)
%         Pt: radiated power (1w)
% Xmin, Xmax: horizontal distance - minimum and maximum (10m, 10000m)
%      Polar: Polarization(E); 1. Vertical(E)
%       					 ; 2. Horizontal(H)
%============================================================================
clear
N=1024;             % the number of points to plot

%--------------------------- input parameters -------------------------------

f=900*10^6;         % frequency
h1=10;				% height of base staion antenna
h2=1.8;             % height of mobile antenna
zt=8;               % transmitter z-coordinate
zr=8;               % receiver z-coordinate
width=30;			% width of street canyon
eg=15.0;            % permittivity of ground
ew=6.0;             %       "         wall
Pt=1;               % total power (radiated power)
Xmin=10;            % minimum x
Xmax=10000;         % maximum x

%------------------------ selection of polarization -------------------------

s=sprintf('\n\nSelect Polarization:\n\t1.Vertical(E) Polarization\n\t2.Horizontal(H) Polarization\n');
Polar=input(s); 	% Polar should be 1 or 2

Vertical=1;		% these assignments are only for readability
Horizontal=2;

if Polar==Vertical,
	A=eg;
	B=1;
	Tstr='Vertical Pol.';
elseif Polar==Horizontal,
	A=1;
	B=ew;
	Tstr='Horizontal Pol.';
else
	disp('Invalid selection for polarization option');
	return;
end

%----------------------------------------------------------------------------

R=logspace(log10(Xmin),log10(Xmax),N); % antenna seperation

C=3*10^8;           % velocity
lamda=C/f;          % wavelength
k=2*pi/lamda;		% wave number

w1=width/2;         % upper wall ('upper' means left wall for prop. dir.)
w2=-w1;             % lower wall ('lower' means right wall for prop. dir.)

dz1=abs(2*w1+zt+zr);	% distance between transmitter and receiver image in w1 dir.
dz2=abs(2*w2+zt+zr);	% distance between transmitter and receiver image in w2 dir.
dhd=abs(h1-h2);			% height difference for path without ground reflection
dhs=h1+h2;				% height difference for path with ground reflection

r1=sqrt(dhd^2+R.^2);  % for direct ray
r2=sqrt(dhs^2+R.^2);  % for ground reflection ray
r3=sqrt(dz1^2+r1.^2); % for upper wall reflection
r4=sqrt(dz2^2+r1.^2); % for lower wall reflection
r5=sqrt(dz1^2+r2.^2); % for upper-ground
r6=sqrt(dz2^2+r2.^2); % for lower-ground

%-------------------------------- gamma2 (G) --------------------------------

CosTheta2=dhs./r2;
temp=sqrt(eg-(1-(CosTheta2.^2)))/A;
Gamma2=(CosTheta2-temp)./(CosTheta2+temp);

%-------------------------------- gamma3 (W1) -------------------------------

CosTheta3=dz1./r3;
temp=sqrt(ew-(1-(CosTheta3.^2)))/B;
Gamma3=(CosTheta3-temp)./(CosTheta3+temp);

%-------------------------------- gamma4 (W2) -------------------------------

CosTheta4=dz2./r4;
temp=sqrt(ew-(1-(CosTheta4.^2)))/B;
Gamma4=(CosTheta4-temp)./(CosTheta4+temp);

%-------------------------------- gamma5w (W1G) -----------------------------

CosTheta5w=dz1./r5;
temp=sqrt(ew-(1-(CosTheta5w.^2)))/B;
Gamma5w=(CosTheta5w-temp)./(CosTheta5w+temp);

%-------------------------------- gamma5g (W1G) -----------------------------

CosTheta5g=dhs./r5;
temp=sqrt(eg-(1-(CosTheta5g.^2)))/A;
Gamma5g=(CosTheta5g-temp)./(CosTheta5g+temp);

%-------------------------------- gamma6w (W2G) -----------------------------

CosTheta6w=dz2./r6;
temp=sqrt(ew-(1-(CosTheta6w.^2)))/B;
Gamma6w=(CosTheta6w-temp)./(CosTheta6w+temp);

%-------------------------------- gamma6g (W2G) -----------------------------

CosTheta6g=dhs./r6;
temp=sqrt(eg-(1-(CosTheta6g.^2)))/A;
Gamma6g=(CosTheta6g-temp)./(CosTheta6g+temp);

%-------------------------------- 6 ray sum ---------------------------------

Pr6Ray=Pt*((lamda/(4*pi))^2)*abs(...
     exp(-j.*k.*r1)./r1...                      % direct path
    +Gamma2.*exp(-j.*k.*r2)./r2...              % ground reflected
    +Gamma3.*exp(-j.*k.*r3)./r3...              % w1 reflected
    +Gamma4.*exp(-j.*k.*r4)./r4...              % w2 reflected
    +Gamma5w.*Gamma5g.*exp(-j.*k.*r5)./r5...    % w1-ground reflected
    +Gamma6w.*Gamma6g.*exp(-j.*k.*r6)./r6).^2;  % w2-ground reflected

%-------------------------------- 2 ray Sum ---------------------------------

Pr2Ray= Pt*((lamda/(4*pi))^2)*abs(...
        exp(-j.*k.*r1)./r1...                   % direct path
        +Gamma2.*exp(-j.*k.*r2)./r2).^2;        % ground reflected

%------------------ main graph, received power vs distance ------------------

figure(1);clf

semilogx(R,10*log10(Pr6Ray),'b-',R,10*log10(Pr2Ray),'r--');
axis([10 10000 -140 -40])
xlabel('Distance (m)' );
ylabel('Path Gain (dB)' );
t=['Path Loss as a Function of Antenna Distance in LOS Street Canyon: ' Tstr];
title(t);

line([580 1120],[-55 -55],'linestyle','-','color','b');
line([580 1120],[-63 -63],'linestyle','--','color','r');

text(1200,-55,'6 Ray Model');
text(1200,-63,'2 Ray Model');

%----------------------------- end of program -------------------------------