%============================================================================
% 2 Path Model for Propagation Over Flat Earth
%
% Input parameters
%
%          f: frequency (900MHz)
%         h1: base staion antenna height (8.7m)
%         h2: mobile antenna height (1.8m)
%         er: dielectric constant for ground (15.0)
% Dmin, Dmax: horizontal antenna seperation - minimum and maxmum (1m, 1000m)
%    RefCoef: ground reflection coefficient
%
%               1. TM  2. TE  3. Gamma=-1
%
%============================================================================

f=900e6;        % frequency=900MHz
h1=8.7;         % height of antenna1
h2=1.8;         % height of antenna2
er=15.0;        % permittivity of ground

Dmin=1;         % minimum distance
Dmax=1000;      % maximum distance

%---------------------- selection of reflection mode ------------------------

s=sprintf('\n\nSelect reflection coefficient:\n\t1.TM\n\t2.TE\n\t3.Gamma=-1\n');
RefCoef=input(s);   % RefCoef should be 1, 2 or 3

TM=1;
TE=2;
MinusOne=3;

%-------------------- calculate reflection coefficient ----------------------

N=1024;         % the number of points to plot
C=3*10^8;       % wave velocity

R=logspace(log10(Dmin),log10(Dmax),N); % antenna seperation
r1=sqrt((h1-h2)^2+R.^2); % direct path
r2=sqrt((h1+h2)^2+R.^2); % ground-reflected path

Pt=1;           % total power (radiated power)
lamda=C/f;      % wavelength
k=2*pi/lamda;   % wave number

CosTheta=(h1+h2)./r2;

if RefCoef==TM,
    temp=sqrt(er-(1-(CosTheta.^2)))/er;
    Gamma=(CosTheta-temp)./(CosTheta+temp);
    Tstr='TM';
elseif RefCoef==TE,
    temp=sqrt(er-(1-(CosTheta.^2)));
    Gamma=(CosTheta-temp)./(CosTheta+temp);
    Tstr='TE';
elseif RefCoef==MinusOne,
    Gamma=-1;
    Tstr='-1';
else
    disp('Invalid selection for reflection coefficient');
    return;
end

%--------------------------- calculate path gain ----------------------------

Pr=Pt*((lamda/(4*pi))^2)*abs(exp(-j.*k.*r1)./r1+Gamma.*exp(-j.*k.*r2)./r2).^2;

%------------------------------ plot the result -----------------------------

figure(1)
semilogx(R,10*log10(Pr));
grid on

xlabel('Distance (m)');
ylabel('Path Gain (dB)');
title(['Path Loss as a Function of Antenna Separation: Ref. Coef = ' Tstr])

%----------------------------- end of program -------------------------------