/*                                                                         */
/*  C++ Program for Numerical Field Calculation of Diffraction past        */ 
/*  Absorbing Screens                                                      */
/*                                                                         */
/*  Program Input                                                          */
/*                                                                         */
/*         - name   : hill.dat, rand.dat or user-defined input file        */
/*         - format : refer data files of hill.dat and rand.dat            */
/*                                                                         */
/*  Program Output                                                         */
/*                                                                         */
/*         - name   : top.dat or screen_nnn.dat (optional)                 */
/*         - format of top.dat                                             */
/*          	<Screen_no> <Height of screen> <Field[dB]> <EPL[dB]>       */
/*         - format of screen_nnn.dat                                      */
/*          	<Height of observing point> <Field[dB]>                    */
/*                                                                         */
/*                                                                         */
/*  Note : 1. 'nnn' indicates the screen number that is specified.         */
/*         2. 'EPL' means Excess Path Loss that is a ratio of the Field    */
/*             value to the free space loss at the observing location.     */
/*                                                                         */
/*                                                                         */
/*                                  By : Hyun Kyu Chung                    */
/*                                       Propagation Lab, EE Dept.         */
/*                                       Polytechnic University            */
/*                                                                         */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <complex.h>
#include <time.h>
#include <math.h>

#define	PI 		3.14159265358979323846
#define	TWOPI 		2.0*PI
#define	THREEPI		3.0*PI
#define	MAXLINE		1000
#define	MAXCHAR		1000

double sgn(double val);
double win_zero_kb(double y,double yc,double w);
complex integral(double Ym,double del,double Yp,double d,double K,complex H,complex H1,double Yc, double f0);

int main()
{
FILE *in, *out;
char line[MAXLINE],in_file[MAXCHAR],junk[MAXCHAR],name[MAXCHAR];
int  i,N,M,P,scrn,p,m,count,OPTION,SCREEN;
double freq, del, Del, yt_m, f0_m, rho;
double lambda, K, Lmax, Wfn, yt, f0, Y;
double *x, *h, *r, Ym;
double XTX, YTX, Hmax, Xmax;
double slope, yline, diff, integ;
complex J,*Hn,*Hn1,*H0,Field,EPL;

time_t thetime =time(NULL);
printf("\n%s \n", ctime(&thetime));
J  = complex(0.0,1.0);

/*                                   */
/* Part I : Reading Input Parameters */
/*                                   */

printf("Enter the name of the input file: ");
scanf("%s", in_file);
in  = fopen(in_file,"r");
if(in == NULL) {
	printf("\nInvalid input file name !\n\n");
   exit(0);
}

count = 0;
while(fgets(line,MAXLINE,in) != NULL) {
	if(strchr("%FDIWS \n",line[0]) != NULL) {
		switch(line[0]) {
			case '%'  : break;
			case '\n' : break;
			case ' '  : break;   
			case 'F'  : sscanf(line,"%s %lf \n", &junk, &freq); break;
			case 'D'  : sscanf(line,"%s %lf \n", &junk, &Del); break;
			case 'I'  : sscanf(line,"%s %lf \n", &junk, &yt_m); break;
			case 'W'  : sscanf(line,"%s %lf \n", &junk, &f0_m); break;
			case 'S'  : sscanf(line,"%s %lf %lf \n", &junk, &XTX, &YTX); break;
		} /* end of switch */
	} /* end of if */
	else if(strchr("+0123456789-",line[0]) != NULL) {
			count++;
	}
} /* end of while */
N = count;
fclose(in);

printf("\n** reading parameters\n\n");
printf("freq  = %f\n",freq);
printf("Del   = %f\n",Del);
printf("yt_m  = %f\n",yt_m);
printf("f0_m  = %f\n",f0_m);
printf("XTx   = %f\n",XTX);
printf("YTx   = %f\n",YTX);
printf("N     = %d\n",N);  

/*                                              */
/* Part II : Reading Distance(x) and Heights(h) */
/*                                              */

x  = new double[N+1];
h  = new double[N+1];
r  = new double[N];
in  = fopen(in_file,"r");
i = 1;
Hmax = 0;
Xmax = 0;
while(fgets(line,MAXLINE,in) != NULL) {
	if(strchr("+0123456789-",line[0]) != NULL) {
		sscanf(line,"%lf %lf",&(x[i]),&(h[i]));
		if (h[i] > Hmax) {
			Hmax = h[i];
			Xmax = x[i];
		}
		i++;
	} /* end of if */
} /* end of while */
fclose(in);

r[0] = x[1] - XTX;
for(i = 1;i < N;i++){
	r[i] = x[i+1] - x[i];
}	
	
/*                                            */
/* Part III : Calculation of extra parameters */
/*                                            */

lambda = 300.0/freq;
K = (2.0*PI)/lambda;
Lmax = x[N]-XTX;
Wfn =sqrt(lambda*Lmax);
yt = yt_m*Wfn;
f0 = f0_m*sqrt(lambda*r[0]);
del = lambda/sqrt(Del);

printf("\n*** extra parameter\n\n");
printf("lambda = %f\n",lambda);
printf("K      = %f\n",K);
printf("Lmax   = %f\n",Lmax);
printf("Wfn    = %f\n",Wfn);
printf("yt     = %f\n",yt); 
printf("f0     = %f\n",f0);
printf("del    = %f\n",del);
printf("Xmax   = %f\n",Xmax);
printf("Hmax   = %f\n",Hmax);

/*                                        */
/* Part IV : Generate Output File         */
/*                                        */

OPTION = 0;
SCREEN = 0;
printf("\nOption of Output File Generation : \n\n");
printf("\t[1] top.dat\n");
printf("\t   (Field vs Screen Number at the top of screen)\n\n");
printf("\t[2] screen_nnn.dat\n");
printf("\t   (Field vs Observing Height at a specific screen)\n\n");
printf("Choose the neumber [1 or 2] ? ");
scanf("%d",&OPTION);

if((OPTION != 1) && (OPTION != 2)) {
	printf("\nInvalid Option\n\n");
   exit(0);
}

switch(OPTION) {
	case 1  : out = fopen("top.dat","w"); break;
	case 2  : printf("Screen Number [1 - %d] ? ",N);
   			 scanf("%d",&SCREEN);
   			 if((SCREEN < 1) && (SCREEN > N)) {
				 	printf("\nInvalid Screen Number !\n");
   			 	exit(0);
   			 }
             sprintf(name,"screen_%03d.dat",SCREEN);
   			 out = fopen(name,"w");
             break;
}

/*                                        */
/* Part V : Fields above the 1st Screen  */
/*                                        */

slope = (h[N]-Hmax)/(x[N]-Xmax);
yline = slope*(x[1]-Xmax)+Hmax;
diff  = yline - h[1];
integ = diff + yt + f0;
P = (int)(integ/del + 0.5);
Hn = new complex[P+1];
for(p = 0;p < P;p++) {
	Y = h[1] + p*del;
	rho = sqrt((Y-YTX)*(Y-YTX)+r[0]*r[0]);
	Hn[p] = exp(-J*K*rho)/sqrt(K*rho);
	Hn[p] = Hn[p] * win_zero_kb(Y,diff+yt,f0);
}
M = P;
printf("\nscreen %03d ..... done\n",1);

switch(OPTION) {
	case 1  :fprintf(out,"%03d\t%f\t%f\t%f\n",1,h[1],20*log10(abs(Hn[0])),0);
   			fflush(out);
   			break;
	case 2  :if(SCREEN == 1) {
   				for(p = 0;p < P;p++) {
						Y = h[1] + p*del;
                 	fprintf(out,"%f\t%f\n",Y,20*log10(abs(Hn[p])));
                  fflush(out);
              	}
				}
            break;
}

/*                                           */
/* Part VI : Fields above the nth Screen  */
/*                                           */


for(scrn = 2;scrn <= N;scrn++) {
	printf("screen %03d ..... ",scrn);
	yline = slope*(x[scrn]-Xmax)+Hmax;
	diff  = yline - h[scrn];
	integ = diff + yt + f0;
	P = (int)(integ/del + 0.5);
	Hn1  = new complex[P+1];
	H0   = new complex[P+1];

	for(p = 0;p <= P;p++) {
		Y = h[scrn]+p*del;
		rho = sqrt((Y-YTX)*(Y-YTX)+(x[scrn]-XTX)*(x[scrn]-XTX));
		H0[p] = exp(-J*K*rho)/sqrt(K*rho);
		Field  = 0;
		for(m = 0;m < M;m++) {
			Ym = h[scrn-1] + m*del;
			Field = Field + integral(Ym,del,Y,r[scrn-1],K,Hn[m],Hn[m+1],diff+yt,f0);
		}
		Hn1[p] = Field;
 	} // end of p

   switch(OPTION) {
		case 1  :EPL = Hn1[0]/H0[0];
      			fprintf(out,"%03d\t%f\t%f\t%f\n",scrn,h[scrn],20*log10(abs(Hn1[0])),20*log10(abs(EPL)));
               fflush(out);
   				break;
		case 2  :if(SCREEN == scrn) {
   					for(p = 0;p < P;p++) {
							Y = h[scrn] + p*del;
               		fprintf(out,"%f\t%f\n",Y,20*log10(abs(Hn1[p])));
                  	fflush(out);
               	}
      			}
					break;
   }

 	delete Hn;
	M = P;
	Hn  = new complex[M+1];

	for(m=0; m <= M; m++) {
		  Hn[m] = Hn1[m];
	}
	delete Hn1,H0;
  	printf("done\n");
 } // end of scrn

 thetime =time(NULL);
 printf("\n%s \n", ctime(&thetime));
 fclose(out);
 return(0);
}

//
// Window function keeping from spurious effect
// - neutralizing to zero
//

double sgn(double val)
{
	if(val >= 0.0)	return(1.0);
	else		return(-1.0);
}

double win_zero_kb(double y,double yc,double w)
{
	double	temp, W_y;

	if (y <= yc)
		return(1.0);
	else if (y >= yc+w)
		return(0.0);
	else {
	    	temp = (y-yc)/w;
    		W_y = .40208+(.49858*cos(PI*temp))+(.09811*cos(TWOPI*temp))+(.00123*cos(THREEPI*temp));
   		return(W_y);
	}
}

complex integral(double Ym,double del,double Yp,double d,double K,complex H,complex H1,double Yc, double f0)
{
	double	rpm, rpm1, Phim, Phim1, Dpm, lambda;
	complex	J, Am, Am1, Coef, Field;

        J = complex(0.0,1.0);
	lambda = TWOPI/K;

	rpm  = sqrt(pow(Ym-Yp,2.0)+pow(d,2.0));
	rpm1 = sqrt(pow(Ym+del-Yp,2.0)+pow(d,2.0));
	Am  = H/sqrt(rpm)*(d/rpm)*win_zero_kb(Ym,Yc,f0);
	Am1 = H1/sqrt(rpm1)*(d/rpm1)*win_zero_kb(Ym+del,Yc,f0);
	Phim  = -K*rpm;
	Phim1 = -K*rpm1;
	Dpm = Phim1-Phim;

	Field = (Am1*exp(J*Phim1)-Am*exp(J*Phim))/Dpm;
	Field = Field+J*(Am1-Am)/Dpm/Dpm*(exp(J*Phim1)-exp(J*Phim));
	Coef = sgn(del)*del*exp(-J*PI/4.0)/sqrt(lambda);
	return(Field*Coef);
}
